import re
import inspect
from typing import (
    Any, 
    Union, 
    Callable, 
    List, 
    Tuple, 
    Set, 
    Dict, 
    get_origin, 
    get_args, 
    _GenericAlias, 
    Literal
)
from functools import wraps
from crealand._apis import _constants
from crealand._utils import _logger_setup

_logger = _logger_setup.setup_logger()

def check_out_of_range(
    func_name, 
    arg_name, 
    arg_value, 
    arg_range_type
):
    cons_dict = arg_range_type.__dict__
    for attr in cons_dict.keys():
        if not attr.startswith("_") and cons_dict[attr] == arg_value:
            return

    raise TypeError(get_out_of_range_error_info(
        func_name, arg_name, arg_value, arg_range_type))

def get_out_of_range_error_info(
    func_name, 
    arg_name, 
    arg_value, 
    arg_range_type, 
    escape=False
):
    cons_name = arg_range_type.__name__
    if escape:
        return re.escape(func_name + '(): The "' + arg_name + 
            '" argument must be in the range of ' + cons_name + 
            ", but got '" + str(arg_value)) + "'"
    else:
        return f'{func_name}(): The "{arg_name}" argument \
must be in the range of {cons_name}, but got \'{arg_value}\''

def check_compare_operator(
    func_name, 
    arg_name, 
    arg_value
):
    cmp_dict = _constants.CompareOperator.__dict__
    for attr in cmp_dict.keys():
        if not attr.startswith("_") and cmp_dict[attr] == arg_value:
            return

    raise TypeError(get_compare_operator_error_info(
        func_name, arg_name, arg_value))

def get_compare_operator_error_info(
    func_name, 
    arg_name, 
    arg_value, 
    escape=False
):
    cmp_dict = _constants.CompareOperator.__dict__
    cmp_opt_list = [cmp_dict[attr] for attr in cmp_dict.keys() 
        if not attr.startswith("_")]
    if escape:
        return re.escape(func_name + '(): The "' + arg_name + 
            '" argument must be in the range of ' + str(cmp_opt_list) + 
            ", but got '" + str(arg_value)) + "'"
    else:
        return f'{func_name}(): The "{arg_name}" argument \
must be in the range of {cmp_opt_list}, but got \'{arg_value}\''

def check_three_dimensions(
    func_name, 
    arg_name, 
    arg_value
):
    if len(arg_value) != 3:
        raise TypeError(get_three_dimensions_error_info(
            func_name, arg_name, arg_value))
    
def get_three_dimensions_error_info(
    func_name, 
    arg_name, 
    arg_value, 
    escape=False
):
    if escape:
        return re.escape(func_name + '(): The "' + arg_name + 
            '" argument must have three dimensions, but got ' 
            + str(arg_value))
    else:
        return f'{func_name}(): The "{arg_name}" argument \
must have three dimensions, but got {arg_value}'

def check_attachment_id(
    func_name, 
    arg_name, 
    arg_value
):
    if len(arg_value) < 1 or len(arg_value) > 2:
        raise TypeError(get_attachment_id_error_info(
            func_name, arg_name, arg_value))
    
def get_attachment_id_error_info(
    func_name, 
    arg_name, 
    arg_value, 
    escape=False
):
    if escape:
        return re.escape(func_name + '(): The "' + arg_name + 
            '" argument can only have one or two elements, but got ' 
            + str(arg_value))
    else:
        return f'{func_name}(): The "{arg_name}" argument \
can only have one or two elements, but got {arg_value}'

def over_args_error_info(
    func_name, 
    func_args_num, 
    given_args_num, 
    escape=False
):
    if escape:
        return re.escape(func_name + "() takes " + str(func_args_num) + 
            " positional arguments but " + str(given_args_num) 
            + " were given")
    else:
        return f"{func_name}() takes {func_args_num} \
positional arguments but {given_args_num} were given"

def miss_args_error_info(
    func_name, 
    miss_args_num, 
    miss_arg_list, 
    escape=False
):
    if escape:
        return re.escape(func_name + "() missing " + str(miss_args_num) + 
            " required positional arguments: " + ", ".join(miss_arg_list))
    else:
        return f"{func_name}() missing {miss_args_num} \
required positional arguments: {', '.join(miss_arg_list)}"

def wrong_type_arg_error_info(
    func_name, 
    arg_name, 
    arg_correct_type, 
    arg_wrong_type, 
    escape=False
):
    if escape:
        return re.escape(func_name + '(): The "' + arg_name + 
            '" argument must be of type ' + str(arg_correct_type) + 
            ", but got " + str(arg_wrong_type))
    else:
        return f'{func_name}(): The "{arg_name}" argument \
must be of type {arg_correct_type}, but got {arg_wrong_type}'

def Handle_point(value: tuple):
    result = {"id": value[0]}
    if len(value) == 2:
        result["name"] = value[1]
    return result

def check_type(func):
    def is_instance_of(value, annotation):
        if annotation is Any:
            return True
        if get_origin(annotation) is Union:
            return any(is_instance_of(value, t) for t in get_args(annotation))
        elif annotation is Callable:
            return callable(value)
        elif get_origin(annotation) in (List, Tuple, Set, Dict,Literal):
            if get_origin(annotation) is Literal:
                return value in get_args(annotation)
            if not isinstance(value, get_origin(annotation)):
                return False
            if not get_args(annotation):  # 如果没有指定元素类型，直接返回 True
                return True
            if get_origin(annotation) is dict:
                key_type, value_type = get_args(annotation)
                return all(is_instance_of(k, key_type) 
                    and is_instance_of(v, value_type) for k, v in value.items())
            else:
                element_type = get_args(annotation)[0]
                return all(is_instance_of(item, element_type) for item in value)
        elif isinstance(annotation, type):
            return isinstance(value, annotation)
        elif isinstance(annotation, _GenericAlias):
            origin = get_origin(annotation)
            args = get_args(annotation)
            if origin is None:
                return False
            if not isinstance(value, origin):
                return False
            if args:
                if origin is dict:
                    key_type, value_type = args
                    return all(is_instance_of(k, key_type) 
                        and is_instance_of(v, value_type) for k, v in value.items())
                elif origin in (list, tuple, set):
                    element_type = args[0]
                    if get_origin(annotation) is Union:
                        return any(is_instance_of(item, element_type) for item in value)
                    else:
                        return all(is_instance_of(item, element_type) for item in value)
            return True
        return False
    
    @wraps(func)
    def inner(*args, **kwargs):
        # 获取原始函数
        original_func = func
        if isinstance(original_func, (staticmethod, classmethod)):
            original_func = original_func.__func__
        
        # 获取函数的类型注解
        annotations = original_func.__annotations__

        # 获取函数的签名信息
        parameters = inspect.signature(original_func).parameters

        # 获取函数的参数名称
        arg_names = list(parameters.keys())

        # 检查位置参数数量
        given_args = list(args) + list(kwargs.values())
        if len(given_args) > len(arg_names):
            raise TypeError(over_args_error_info(
                func_name=original_func.__name__, 
                func_args_num=len(arg_names), 
                given_args_num=len(args)
            ))

        # 检查是否有未传递的必需参数
        required_params = [name for name in arg_names 
            if parameters[name].default == inspect.Parameter.empty]
        if len(given_args) < len(required_params):
            raise TypeError(miss_args_error_info(
                func_name=original_func.__name__, 
                miss_args_num=len(required_params) - len(given_args), 
                miss_arg_list=required_params[len(given_args):]
            ))

        # 检查位置参数
        for i, arg in enumerate(given_args):
            param_name = arg_names[i]
            if param_name in annotations:
                annotation = annotations[param_name]
                if not is_instance_of(arg, annotation):
                    raise TypeError(wrong_type_arg_error_info(
                        func_name=original_func.__name__, 
                        arg_name=param_name, 
                        arg_correct_type=annotation, 
                        arg_wrong_type=type(arg)
                    ))

        return original_func(*args, **kwargs)

    return inner

def raise_error(func,err,data):
    if err > 0:
        raise Exception(f"Error occurred:func {func} {err}, {data}")
    else:
        _logger.info(f"Error occurred:func {func} {err}, {data}")


from crealand._core.bridge import _interface
from crealand._apis import _subscribe_event
from crealand._utils import _utils
from typing import (
    Callable as _Callable, 
    Union as _Union
)

_decibel_val = 0

# 分贝值
def get_decibel_value():
    global _decibel_val
    return _decibel_val

# 开始识别
def start_decibel_recognition():

    def cb_wrapper(err, data):
        if err is None:
           global _decibel_val
           _decibel_val = data["data"]
        else:
           _utils.raise_error("start_decibel_recognition",err,data)

    _subscribe_event.onSensorSoundEvent("==", "", cb_wrapper)

# 结束识别
def stop_decibel_recognition():
    _interface.call_api_async(
        "web-ide", 
        "api.openDecibelDetectionPage", 
        [{"type":"end"}]
    )

@_utils.check_type
def onEventDecibel(decibel_value: _Union[int, float], cb: _Callable):
    decibel_value = max(0, min(decibel_value, 150))
    def cb_wrapper(err, data):
        if err is None:
           global _decibel_val
           _decibel_val = data["data"]
           cb()
        else:
           _utils.raise_error("onEventDecibel", err, data)


    _subscribe_event.onSensorSoundEvent(">", decibel_value, cb_wrapper)


# 获取虚拟相机
@_utils.check_type
def virtual_camera(runtime_id: int,status: bool=True):
    runtime_id = max(0, runtime_id)
    _interface.call_api(
        "unity", 
        "unity.camera.openVirsual", 
        [runtime_id,status]
    )



from crealand._core.bridge import _interface
from crealand._apis import _constants
from crealand._utils import _utils
from typing import Union as _Union 

class Dialogue:
    # 立绘对话 获取选项
    option_value = ""
    @staticmethod
    def get_option_value():
        return Dialogue.option_value

    # 立绘对话 初始化
    @staticmethod
    def init():
        _interface.call_api(
            "web-ide", 
            "api.prepareDialogBoard", 
            [{}]
        )
        

    # 立绘对话 显示
    @_utils.check_type
    @staticmethod
    def set_dialogue(
        obj_name: str,
        content: str,
        res_id: str="",
        volume: str=_constants.Volume.MEDIUM
    ):
        _utils.check_out_of_range("set_dialogue", "volume", 
            volume, _constants.Volume)
        _interface.call_api(
            "web-ide",
            "api.showDialog",
            [
                {
                    "speaker": obj_name,
                    "type": volume,
                    "txt": content,
                    "voiceId": "",
                    "pythonOssId":res_id,
                }
            ],
        )
    
    @_utils.check_type
    @staticmethod
    def set_dialogue_tone(
        obj_name: str,
        content: str,
        res_id: str="",
        tone: str="",
        volume: str=_constants.Volume.MEDIUM,
    ):
        if tone != "":
            _utils.check_out_of_range("set_dialogue_tone", "tone", 
                tone, _constants.Tone)
        _utils.check_out_of_range("set_dialogue_tone", "volume", 
            volume, _constants.Volume)
        _interface.call_api(
            "web-ide",
            "api.showDialog",
            [
                {
                    "speaker": obj_name,
                    "type": volume,
                    "txt": content,
                    "voiceId": "",
                    "imgId":tone,
                    "pythonOssId":res_id,
                }
            ],
        )

    # 立绘对话 设置选项
    @_utils.check_type
    @staticmethod
    def set_option(
        content: str, 
        opt_name: str=_constants.OptionName.OPTION01
    ):
        _utils.check_out_of_range("set_option", "opt_name", 
            opt_name, _constants.OptionName)
        options = {}
        options[opt_name] = content
        _interface.call_api(
            "web-ide",
            "api.setDialogOptions",
            [{"options": options}],
        )

    # 立绘对话选项 显示
    @_utils.check_type
    @staticmethod
    def set_option_show(is_show: bool=True):
        Dialogue.option_value = _interface.call_api(
            "web-ide", 
            "api.toggleDialogOptions", 
            [{"show": is_show}]
        )
        
    #  立绘对话 显示
    @_utils.check_type
    @staticmethod
    def show(is_show: bool=True):
        if not is_show:
           Dialogue.option_value = ""
            
        _interface.call_api(
            "web-ide", 
            "api.toggleDialogBoard", 
            [{"show": is_show}]
        )


class HelpPanel:
    # 帮助面板 初始化
    @staticmethod
    def init():
        _interface.call_api("web-ide", "api.prepareHelpboard", [{}])

    # 帮助面板 设置标题
    @_utils.check_type
    @staticmethod
    def set_tips(title: str, res_id: str=""):
        _interface.call_api(
            "web-ide",
            "api.addHelpItem",
            [
                {
                    "title": title,
                    "pythonOssId": res_id,
                }
            ],
        )

    # 帮助面板 显示
    @_utils.check_type
    @staticmethod
    def show(is_show: bool=True):
        _interface.call_api(
            "web-ide",
            "api.toggleHelpboard",
            [
                {
                    "show": is_show,
                }
            ],
        )


class TaskPanel:

    # 任务面板 设置标题
    @_utils.check_type
    @staticmethod
    def set_task(title: str, nickname: str):
        _interface.call_api(
            "web-ide",
            "api.createTaskboard",
            [
                {
                    "title": title,
                    "alias": nickname,
                }
            ],
        )

    # 任务面板 设置任务项
    @_utils.check_type
    @staticmethod
    def set_task_progress(
        task_name: str, 
        subtasks_content: str, 
        completed_tasks: int, 
        total_tasks: int
    ):
        total_tasks = max(1, total_tasks)
        completed_tasks = max(0, min(completed_tasks, total_tasks))
        _interface.call_api(
            "web-ide",
            "api.setTaskboard",
            [
                {
                    "alias": task_name,
                    "taskName": subtasks_content,
                    "process": [completed_tasks, total_tasks],
                }
            ],
        )

    # 任务面板 显示
    @_utils.check_type
    @staticmethod
    def set_task_show(task_name: str, is_show: bool=True):
        _interface.call_api(
            "web-ide",
            "api.toggleTaskboard",
            [{"alias": task_name, "show": is_show}],
        )


class Speak:
    # 说
    @_utils.check_type
    @staticmethod
    def text(runtime_id: int, content: str, time: _Union[int, float]=2):
        runtime_id = max(0, runtime_id)
        if time != -1:
            time = max(0, time)
        _interface.call_api_async(
            "unity", 
            "unity.actor.speak", 
            [runtime_id, content, time]
        )

    # 说-img
    @_utils.check_type
    @staticmethod
    def image(runtime_id: int, res_id: str, time: _Union[int, float]=2):
        runtime_id = max(0, runtime_id)
        if time != -1:
            time = max(0, time)
        _interface.call_api_async(
           "unity", 
           "unity.actor.speakImage", 
           [runtime_id, res_id, time]
        )


class Interactive:
    # 提示面板显示
    @_utils.check_type
    @staticmethod
    def set_tip_show(option: str=_constants.ResultType.START):
        _utils.check_out_of_range("set_tip_show", "option", 
            option, _constants.ResultType)
        _interface.call_api(
            "web-ide",
            "api.showTipboardResult",
            [
                {
                    "result": option,
                }
            ],
        )

    # 提示面板显示
    @_utils.check_type
    @staticmethod
    def toast(
        content: str,
        position: str=_constants.ToastPosition.TOP,
        state: str=_constants.ToastState.DYNAMIC,
    ):
        _utils.check_out_of_range("toast", "position", 
            position, _constants.ToastPosition)
        _utils.check_out_of_range("toast", "state", 
            state, _constants.ToastState)
        _interface.call_api_async(
            "web-ide",
            "api.toast",
            [
                {
                    "position": position,
                    "mode": state,
                    "txt": content,
                }
            ],
            
        )



import time as _time
from crealand._core.bridge import _interface
from crealand._apis import _subscribe_event, _constants
from crealand._utils import _utils
from typing import Callable as _Callable

#收到广播事件
@_utils.check_type
def onBroadcastEvent(info: str, cb: _Callable):
    _subscribe_event.onBroadcastEvent(info,cb)

#发送广播事件
@_utils.check_type
def send_broadcast(info: str):
    _subscribe_event.sendBroadcast(info)

#对象进入/离开判定区域事件
@_utils.check_type
def onAreaObjectEvent(
    runtime_id: int, 
    action: int, 
    area_id: int, 
    cb: _Callable
):
    _utils.check_out_of_range("onAreaObjectEvent", "action", 
                              action, _constants.ActionType)
    runtime_id = max(0, runtime_id)
    area_id = max(0, area_id)
    def cb_wrapper(err, data):
        if err is None:
            cb()
        else:
            _utils.raise_error("onAreaObjectEvent", err, data)
    _subscribe_event.onAreaObjectEvent(runtime_id, action, 
        area_id, cb_wrapper)

#分类进入/离开判定区域事件
@_utils.check_type
def onAreaClassEvent(
    config_id: str, 
    action: int, 
    area_id: int, 
    cb: _Callable
):
    _utils.check_out_of_range("onAreaClassEvent", "action", 
        action, _constants.ActionType)
    area_id = max(0, area_id)
    def cb_wrapper(err, data):
        if err is None:
            cb()
        else:
            _utils.raise_error("onAreaClassEvent", err, data)
    _subscribe_event.onAreaClassEvent(config_id, action, 
        area_id, cb_wrapper)

# 验证按键是否按下
@_utils.check_type
def keypress_state(button: int):
    _utils.check_out_of_range("keypress_state", "button", 
        button, _constants.KeyType)
    _time.sleep(_constants.SLEEP_TIME)
    return _interface.call_api(
        "unity", 
        "unity.input.verifyKeyCodeState", 
        [button, _constants.KeyPress.KEY_PRESS]
    )

# 鼠标键盘事件
@_utils.check_type
def onKeyEvent(action: int, button: int, cb: _Callable):
    _utils.check_out_of_range("onKeyEvent", "action", 
        action, _constants.KeyActiveType)
    _utils.check_out_of_range("onKeyEvent", "button", 
        button, _constants.KeyType)
    def cb_wrapper(err, data):
        if err is None:
            cb()
        else:
            _utils.raise_error("onKeyEvent", err, data)
    _subscribe_event.onKeyEvent(action, button, cb_wrapper)



from crealand._core.bridge import _interface
from crealand._apis import _constants
from crealand._utils import _utils
from typing import (
    List as _List, 
    Union as _Union, 
    Literal as _Literal
)

# 信息
class Info:

    # 别名对象id
    @_utils.check_type
    @staticmethod
    def get_alias_id(nickname: str):
        result = _interface.call_api(
            "unity", 
            "unity.alias.getAlias", 
            [nickname]
        )
        return result

    # 获取configID的对象id
    @_utils.check_type
    @staticmethod
    def get_object_id(runtime_id: int) -> int:
        runtime_id = max(0, runtime_id)
        result = _interface.call_api(
            "unity",
            "unity.actor.getConfigID",
            [runtime_id]
        )
        return result

    # 获取对象的空间坐标
    @_utils.check_type
    @staticmethod
    def get_object_coordinates(runtime_id: int):
        runtime_id = max(0, runtime_id)
        result = _interface.call_api(
            "unity", 
            "unity.actor.getCoordinate", 
            [runtime_id]
        )
        return result

    # 获取判定区域中的对象id
    @_utils.check_type
    @staticmethod
    def get_id_in_area(area_id: int, config_ids: _List[str]):
        area_id = max(0, area_id)
        result = _interface.call_api(
            "unity",
            "unity.editableTrigger.getContentRuntimeIds",
            [area_id, config_ids]
        )
        return result

    # 获取空间坐标某个轴的值
    @_utils.check_type
    @staticmethod
    def get_spatial_coordinates(
        coordinate: _List[_Union[int, float]], 
        axis: _Literal["X", "Y", "Z"]="X"
    ):
        _utils.check_three_dimensions("get_spatial_coordinates", "coordinate", 
            coordinate)
        AXIS = {"X": 0, "Y": 1, "Z": 2}
        return coordinate[AXIS[axis]]

    # 获取对象的运动方向向量
    @_utils.check_type
    @staticmethod
    def get_motion_vector(runtime_id: int):
        runtime_id = max(0, runtime_id)
        result = _interface.call_api(
            "unity", 
            "unity.character.getMoveDirection", 
            [runtime_id]
        )
        return result


class Camera:
    default_id = ""

    # 获取相机ID
    @staticmethod
    def get_default_id():
        Camera.default_id = _interface.call_api(
            "unity", 
            "unity.camera.getDefaultID", 
            []
        )
        return Camera.default_id

    # 获取空间坐标
    @_utils.check_type
    @staticmethod
    def get_object_coordinates(runtime_id: int):
        runtime_id = max(0, runtime_id)
        result = _interface.call_api(
            "unity", 
            "unity.actor.getCoordinate", 
            [runtime_id]
        )
        return result

    # 相机移动
    @_utils.check_type
    @staticmethod
    def move_to(
        time: _Union[int, float], 
        coordinate: _List[_Union[int, float]], 
        block: bool=False
    ):
        time = max(0, time)
        _utils.check_three_dimensions("move_to", "coordinate", coordinate)
        _interface.call_api(
            "unity",
            "unity.camera.moveTo",
            [Camera.default_id, time, coordinate, block]
        )

    # 调整FOV
    @_utils.check_type
    @staticmethod
    def adjust_FOV(time: _Union[int, float]=1, fov: _Union[int, float]=80):
        time = max(0, time)
        fov = max(60, min(fov, 120))
        _interface.call_api_async(
            "unity",
            "unity.camera.adjustFOV",
            [Camera.default_id, time, fov]
        )

    # 相机锁定朝向并移动
    @_utils.check_type
    @staticmethod
    def move_while_looking(
        coordinate_1: _List[_Union[int, float]],
        time: _Union[int,float]=1,
        coordinate_2: _List[_Union[int, float]]=[0, 0, 1],
        block: bool=False
    ):
        _utils.check_three_dimensions("move_while_looking", "coordinate_1", 
            coordinate_1)
        time = max(0, time)
        _utils.check_three_dimensions("move_while_looking", "coordinate_2", 
            coordinate_2)
        _interface.call_api_async(
            "unity",
            "unity.camera.moveWhileLooking",
            [Camera.default_id, time, coordinate_2, coordinate_1, block]
        )

    # 获取相机坐标
    @staticmethod
    def get_camera_coordinate() -> _List[_Union[int, float]]:
        result = Camera.get_object_coordinates(Camera.default_id)
        return result

    # 相机朝向
    @_utils.check_type
    @staticmethod
    def look_at(coordinate: _List[_Union[int, float]]):
        _utils.check_three_dimensions("look_at", "coordinate", coordinate)
        _interface.call_api_async(
            "unity",
            "unity.camera.lookAt",
            [Camera.default_id, coordinate]
        )

    # 相机跟随
    @_utils.check_type
    @staticmethod
    def follow_target(
        runtime_id: int, 
        distance: _Union[int, float]=10, 
        is_rotate: bool=True
    ):
        runtime_id = max(0, runtime_id)
        _interface.call_api_async(
            "unity",
            "unity.camera.followTarget",
            [Camera.default_id, runtime_id, distance, is_rotate]
        )

    # 相机结束跟随
    @staticmethod
    def end_follow_target():
        _interface.call_api_async(
            "unity",
            "unity.camera.stopFollowing",
            [Camera.default_id]
        )

    # 相机 滤镜
    @_utils.check_type
    @staticmethod
    def filters(filter_name: int=_constants.FilterStyle.FOG, state: bool=True):
        _utils.check_out_of_range("filters", "filter_name", 
            filter_name, _constants.FilterStyle)
        _interface.call_api_async(
            "unity",
            "unity.camera.openEffect",
            [Camera.default_id, filter_name, state]
        )


class Motion:
    # 创建对象
    @_utils.check_type
    @staticmethod
    def create_object_coordinate(
        config_id: str, 
        coordinate: _List[_Union[int,float]]=[0, 0, 1]
    ):
        _utils.check_three_dimensions("create_object_coordinate", "coordinate", coordinate)
        result = _interface.call_api(
            "unity",
            "unity.actor.createObject",
            [config_id, coordinate]
        )
        return result

    # 测距
    @_utils.check_type
    @staticmethod
    def ray_ranging(
        runtime_id: int, 
        attachment_id: tuple=(_constants.HangPointType.LEFT_FRONT_WHEEL,)
    ):
        runtime_id = max(0, runtime_id)
        _utils.check_attachment_id("ray_ranging", "attachment_id", 
            attachment_id)
        _utils.check_out_of_range("ray_ranging", "attachment_id", 
            attachment_id[0], _constants.HangPointType)
        result = _interface.call_api(
            "unity",
            "unity.actor.rayRanging",
            [runtime_id, _utils.Handle_point(attachment_id), 20]
        )
        return result

    # 移动
    @_utils.check_type
    @staticmethod
    def move_to(
        runtime_id: int, 
        coordinate: _List[_Union[int, float]]=[0, 0, 1]
    ):
        runtime_id = max(0, runtime_id)
        _utils.check_three_dimensions("move_to", "coordinate", coordinate)
        _interface.call_api(
            "unity",
            "unity.actor.setObjectPosition",
            [runtime_id, coordinate]
        )

    # 朝向
    @_utils.check_type
    @staticmethod
    def face_towards(
        runtime_id: int, 
        coordinate: _List[_Union[int, float]]=[0, 0, 1]
    ):
        runtime_id = max(0, runtime_id)
        _utils.check_three_dimensions("face_towards", "coordinate", coordinate)
        _interface.call_api(
            "unity",
            "unity.actor.setObjectTowardPosition",
            [runtime_id, coordinate]
        )

    # 前进
    @_utils.check_type
    @staticmethod
    def move_forward(
        runtime_id: int, 
        speed: int=1, 
        distance: _Union[int, float]=3, 
        block: bool=False
    ):
        runtime_id = max(0, runtime_id)
        speed = max(1, min(speed, 5))
        _interface.call_api(
            "unity",
            "unity.actor.moveForwardByDistance",
            [runtime_id, distance, abs(distance / speed), block]
        )

    # 对象旋转
    @_utils.check_type
    @staticmethod
    def rotate(
        runtime_id: int, 
        time: _Union[int, float]=1, 
        angle: _Union[int, float]=90, 
        block: bool=False
    ):
        runtime_id = max(0, runtime_id)
        time = max(time, 0)
        _interface.call_api(
            "unity",
            "unity.character.rotateUpAxisByAngle",
            [runtime_id, angle, time, block]
        )

    # 云台旋转 & 机械臂旋转
    @_utils.check_type
    @staticmethod
    def ptz(runtime_id: int, angle: _Union[int, float]=90, block: bool=False):
        runtime_id = max(0, runtime_id)
        _interface.call_api(
            "unity",
            "unity.actor.rotatePTZUpAxisByAngle",
            [runtime_id, angle, abs(angle) / 30, block]
        )

    # 播放动作
    @_utils.check_type
    @staticmethod
    def action(runtime_id: int, action: str, block: bool=False):
        runtime_id = max(0, runtime_id)
        _interface.call_api(
            "unity",
            "unity.actor.playAnimation",
            [runtime_id, action, block]
        )

    # # 将对象吸附到挂点
    @_utils.check_type
    @staticmethod
    def attach_to(
        absorbed_runtime_id: int, 
        absorb_runtime_id: int, 
        attachment_id: tuple
    ):
        absorbed_runtime_id = max(0, absorbed_runtime_id)
        absorb_runtime_id = max(0, absorb_runtime_id)
        _utils.check_attachment_id("attach_to", "attachment_id", 
            attachment_id)
        _utils.check_out_of_range("attach_to", "attachment_id", 
            attachment_id[0], _constants.HangPointType)
        _interface.call_api(
            "unity",
            "unity.actor.attach",
            [
                absorbed_runtime_id, 
                absorb_runtime_id, 
                _utils.Handle_point(attachment_id)
            ]
        )

    # 绑定挂点
    @_utils.check_type
    @staticmethod
    def bind_to_object_point(
        runtime_id_1: int,
        attachment_id_1: tuple, 
        runtime_id_2: int,
        attachment_id_2: tuple
    ):
        runtime_id_1 = max(0, runtime_id_1)
        _utils.check_attachment_id("bind_to_object_point", "attachment_id_1", 
            attachment_id_1)
        _utils.check_out_of_range("bind_to_object_point", "attachment_id_1", 
            attachment_id_1[0], _constants.HangPointType)
        runtime_id_2 = max(0, runtime_id_2)
        _utils.check_attachment_id("bind_to_object_point", "attachment_id_2", 
            attachment_id_2)
        _utils.check_out_of_range("bind_to_object_point", "attachment_id_2", 
            attachment_id_2[0], _constants.HangPointType)
        _interface.call_api(
            "unity",
            "unity.actor.bindAnchor",
            [
                runtime_id_1, 
                _utils.Handle_point(attachment_id_1), 
                runtime_id_2, 
                _utils.Handle_point(attachment_id_2)
            ]
        )

    # 解除绑定
    @_utils.check_type
    @staticmethod
    def detach(runtime_id: int):
        runtime_id = max(0, runtime_id)
        _interface.call_api(
            "unity",
            "unity.actor.detach",
            [runtime_id]
        )

    # 向画面空间前进
    @_utils.check_type
    @staticmethod
    def move_towards_screen_space(
        runtime_id: int, 
        speed: int=1, 
        direction: _List[_Union[int, float]]=[0, 0, 1]
    ):
        runtime_id = max(0, runtime_id)
        speed = max(1, min(speed, 5))
        _utils.check_three_dimensions("move_towards_screen_space", "direction", 
            direction)
        _interface.call_api(
            "unity",
            "unity.actor.moveByVelocity",
            [runtime_id, 2, speed, direction]
        )

    @_utils.check_type
    @staticmethod
    def get_motion_vector(runtime_id: int):
        runtime_id = max(0, runtime_id)
        result = _interface.call_api(
            "unity", 
            "unity.character.getMoveDirection", 
            [runtime_id]
        )
        return result

    # 旋转运动方向向量
    @_utils.check_type
    @staticmethod
    def rotate_to_direction(
        runtime_id: int, 
        angle: _Union[int, float]=0, 
        direction: _List[_Union[int, float]]=[0, 0, 1]
    ):
        runtime_id = max(0, runtime_id)
        _utils.check_three_dimensions("rotate_to_direction", "direction", 
            direction)
        _interface.call_api(
            "unity",
            "unity.character.rotateUpAxisByDirection",
            [runtime_id, angle, direction, 0]
        )

    # 停止运动
    @_utils.check_type
    @staticmethod
    def stop(runtime_id: int):
        runtime_id = max(0, runtime_id)
        _interface.call_api_async(
            "unity",
            "unity.character.stop",
            [runtime_id],
        )

    # 设置别名
    @_utils.check_type
    @staticmethod
    def create_object(
        config_id: str,
        nickname: str,
        coordinate: _List[_Union[int, float]]=[0, 0, 1],
    ):
        _utils.check_three_dimensions("create_object", "coordinate", coordinate)
        _interface.call_api_async(
            "unity",
            "unity.alias.setAlias",
            [
                nickname,
                Motion.create_object_coordinate(config_id, coordinate),
            ],
        )

    # 销毁对象
    @_utils.check_type
    @staticmethod
    def destroy(runtime_id: int):
        runtime_id = max(0, runtime_id)
        _interface.call_api_async(
            "unity",
            "unity.alias.destoryObject",
            [runtime_id]
        )

    # 上升
    @_utils.check_type
    @staticmethod
    def rise(
        runtime_id: int, 
        speed: int=3, 
        height: _Union[int, float]=10, 
        block: bool=False
    ):
        runtime_id = max(0, runtime_id)
        speed = max(1, min(speed, 5))
        _interface.call_api(
            "unity",
            "unity.character.moveUpByDistance",
            [runtime_id, height, abs(height/speed), block],
        )
    # 降落
    @_utils.check_type
    @staticmethod
    def landing(runtime_id: int):
        runtime_id = max(0, runtime_id)
        _interface.call_api(
            "unity",
            "unity.character.land",
            [runtime_id, 3],
        )

    # 获取离自身距离的坐标
    @_utils.check_type
    @staticmethod
    def get_object_local_position(
        runtime_id: int, 
        coordinate: _List[_Union[int, float]]=[0, 0, 1], 
        distance: _Union[int,float]=0
    ):
        runtime_id = max(0, runtime_id)
        _utils.check_three_dimensions("get_object_local_position", "coordinate", coordinate)
        result = _interface.call_api(
            "unity",
            "unity.actor.getObjectLocalPosition",
            [runtime_id, coordinate, distance],
        )
        return result

    # 移动到指定坐标
    @_utils.check_type
    @staticmethod
    def move_by_point(
        runtime_id: int, 
        time: _Union[int,float]=1, 
        coordinate: _List[_Union[int, float]]=[0, 0, 1], 
        block: bool=False
    ):
        runtime_id = max(runtime_id, 0)
        time = max(time, 0)
        _utils.check_three_dimensions("move_by_point", "coordinate", coordinate)
        _interface.call_api(
            "unity",
            "unity.actor.moveByPoint",
            [runtime_id, time, coordinate, block]
        )

    # 绕坐标轴旋转
    @_utils.check_type
    @staticmethod
    def rotate_by_origin_and_axis(
        runtime_id: int,
        time: _Union[int,float]=2,
        point_1: int=_constants.AxisType.LOCAL,
        coordinate_1: _List[_Union[int, float]]=[0, 0, 0],
        point_2: int=_constants.AxisType.LOCAL,
        coordinate_2: _List[_Union[int, float]]=[0, 0, 1],
        angle: _Union[int,float]=90,
        block: bool=False
    ):
        runtime_id = max(runtime_id, 0)
        time = max(time, 0)
        _utils.check_out_of_range("rotate_by_origin_and_axis", "point_1", 
            point_1, _constants.AxisType)
        _utils.check_three_dimensions("rotate_by_origin_and_axis", "coordinate_1", 
            coordinate_1)
        _utils.check_out_of_range("rotate_by_origin_and_axis", "point_2", 
            point_2, _constants.AxisType)
        _utils.check_three_dimensions("rotate_by_origin_and_axis", "coordinate_2", 
            coordinate_2)
        _interface.call_api(
            "unity",
            "unity.actor.rotateByOringinAndAxis",
            [
                runtime_id,
                coordinate_1,
                point_1,
                coordinate_2,
                point_2,
                angle,
                time,
                block 
            ]
        )


class Property:
    # 新增自定义属性
    @_utils.check_type
    @staticmethod
    def add_attr(runtime_id: int, attr_name: str, attr_value: str):
        runtime_id = max(runtime_id, 0)
        _interface.call_api(
            "unity",
            "unity.actor.addCustomProp",
            [runtime_id, attr_name, attr_value]
        )

    # 删除自定义属性
    @_utils.check_type
    @staticmethod
    def del_attr(runtime_id: int, attr_name: str):
        runtime_id = max(runtime_id, 0)
        _interface.call_api(
            "unity",
            "unity.actor.delCustomProp",
            [runtime_id, attr_name]
        )

    # 修改自定义属性
    @_utils.check_type
    @staticmethod
    def set_attr(runtime_id: int, attr_name: str, attr_value: str):
        runtime_id = max(runtime_id, 0)
        _interface.call_api(
            "unity",
            "unity.actor.setCustomProp",
            [runtime_id, attr_name, attr_value],
        )

    # 获取自定义属性的值
    @_utils.check_type
    @staticmethod
    def get_value(runtime_id: int, attr_name: str):
        runtime_id = max(runtime_id, 0)
        result = _interface.call_api(
            "unity",
            "unity.actor.getCustomProp",
            [runtime_id, attr_name],
        )
        return result

    # 获取自定义属性组中某一项的值
    @_utils.check_type
    @staticmethod
    def get_value_by_idx(runtime_id: int, index: int=1):
        runtime_id = max(runtime_id, 0)
        result= _interface.call_api(
            "unity",
            "unity.actor.getCustomPropValueByIdx",
            [runtime_id, index],
        )
        return result

    # 获取自定义属性组中某一项的名称
    @_utils.check_type
    @staticmethod
    def get_key_by_idx(runtime_id: int, index: int=1):
        runtime_id = max(runtime_id, 0)
        result=_interface.call_api(
            "unity",
            "unity.actor.getCustomPropKeyByIdx",
            [runtime_id, index],
        )
        return result


class Show:
    # 3d文本-RGB
    @_utils.check_type
    @staticmethod
    def set_3D_text_status_rgb(
        runtime_id: int, 
        rgb: _List[int] = [255, 255, 255], 
        size: int = 30, 
        text: str = "文本"
    ):
        runtime_id = max(runtime_id, 0)
        _utils.check_three_dimensions("set_3D_text_status_rgb", "rgb", rgb)
        for i in range(3):
            rgb[i] = max(0, min(255, rgb[i]))

        size = max(0, min(30, size))
        _interface.call_api(
            "unity",
            "unity.building.set3DTextStatus",
            [runtime_id, rgb, size, text],
        )



from crealand._core.bridge import _interface
from crealand._apis import _subscribe_event, _constants
from crealand._utils import _logger_setup, _utils
from typing import Callable as _Callable

_logger = _logger_setup.setup_logger()

# 人脸识别
# 机器学习
# 数字识别


class Figure:

    _digit=""
    # 打开手写数字识别教学页面
    @staticmethod
    def open_teach_page():
        _interface.call_api(
            "web-ide", 
            "api.openDigitRecognitionTeachingPage", 
            [{"name":""}]
        )

    # 打开神经网络教学页面
    @staticmethod
    def open_NN_teach_page():
        _interface.call_api(
            "web-ide", 
            "api.openNeuralNetworkTeachingPage", 
            [{"type":""}]
        )

    # 开始手写数字识别
    @staticmethod
    def start_digital_recognition():
        # 识别返回的结果需要设置保存
        # call_api_async("web-ide","api.digitRecognition",[{"type":"start"}])
        def on_result(err, data):
            if err is None:
                if data and data.get("data") and data["data"] != "":
                    Figure._digit = int(data["data"])
            else:
                _logger.error(f"Error occurred: {err},{data}")

        _subscribe_event.onAIFigureEvent("", on_result)

    # 结束手写数字识别
    @staticmethod
    def stop_digital_recognition():
        Figure._digit =""
        _interface.call_api_async(
            "web-ide", 
            "api.digitRecognition", 
            [{"type":"end"}]
        )

    # 数字识别结果
    @staticmethod
    def get_figure_value():
        return Figure._digit

    # 清除数字识别结果
    @staticmethod
    def clear_figure_value():
        Figure._digit =""
        _interface.call_api(
            "web-ide", 
            "api.digitRecognitionClear", 
            [{"type":""}]
        )

    @_utils.check_type
    @staticmethod
    def onAIFigureEvent(number: int, cb: _Callable):
        # 识别返回的结果需要设置保存
        number = max(0, min(number, 9))
        # call_api_async("web-ide","api.digitRecognition",[{"type":"start"}])
        def on_result(err, data):
            if err is None:
                if data and data.get("data") and data["data"] != "":
                    Figure._digit = int(data["data"])
                    cb()
            else:
                _logger.error(f"Error occurred: {err},{data}")

        _subscribe_event.onAIFigureEvent(number, on_result)


# 手势识别
class Gesture:
    _direction=""
    # 开始手势识别 并等待结束
    @staticmethod
    def start_gesture_recognition():
        def on_result(err, data):
            if err is None:
                if data and data.get("data") and data["data"] != "":
                    Gesture._direction = data["data"]
            else:
                _logger.error(f"Error occurred: {err},{data}")

        _subscribe_event.onAIGestureEvent("", on_result)

    # 结束手势识别
    @staticmethod
    def stop_gesture_recognition():
        Gesture._direction=""
        _interface.call_api_async(
            "web-ide", 
            "api.gestureRecognition", 
            [{"type":"end"}]
        )

    # 当前手势识别结果为
    @staticmethod
    def get_gesture_value():
        return Gesture._direction

    # 帽子积木块
    @_utils.check_type
    @staticmethod
    def onAIGestureEvent(direction:str,cb:_Callable):
        _utils.check_out_of_range("onAIGestureEvent", "direction", 
            direction, _constants.Direction)
        def on_result(err, data):
            if err is None:
                if data and data.get("data") and data["data"] != "":
                    Gesture._direction = data["data"]
                    cb()
            else:
                _logger.error(f"Error occurred: {err},{data}")

        _subscribe_event.onAIGestureEvent(direction, on_result)

# 语音识别
class Voice:
    _text=""
    # 开始语音识别
    @staticmethod
    def start_voice_recognition():
        def on_result(err, data):
            if err is None:
                if data and data.get("data") and data["data"] != "":
                    Voice._text = data["data"]
            else:
                _logger.error(f"Error occurred: {err},{data}")

        _subscribe_event.onAIAsrEvent("", on_result)


    # 结束识别
    @staticmethod
    def stop_voice_recognition():
        Voice._text =""
        _interface.call_api(
            "web-ide", 
            "api.openVoiceRecognition", 
            [{"type":"end"}]
        )

    # 语音识别结果
    @staticmethod
    def get_voice_value():
        return Voice._text


    # 打开语音识别教学页面
    @staticmethod
    def open_voice_teach_page():
        _interface.call_api(
            "web-ide", 
            "api.openVoiceRecognitionTeachingPage", 
            [{"name":""}]
        )

    # 帽子积木块
    @_utils.check_type
    @staticmethod
    def onAIAsrEvent(text: str, cb: _Callable):
        def on_result(err, data):
            if err is None:
                if data and data.get("data") and data["data"] != "":
                    Voice._text = data["data"]
                    cb()
            else:
                _logger.error(f"Error occurred: {err},{data}")

        _subscribe_event.onAIAsrEvent(text, on_result)





import time as _time
from crealand._core.bridge import _interface
from crealand._apis import _subscribe_event, _constants
from typing import (
    Callable as _Callable, 
    Union as _Union
)
from crealand._utils import _utils

# 超声波传感器

class Ultrasonic:
    _sensors = {}

    # 前端处理传感器信息绑定
    @_utils.check_type
    @staticmethod
    def add_sensor(sensor: str, runtime_id: int, attachment_id: tuple) -> None:
        runtime_id = max(0, runtime_id)
        _utils.check_attachment_id("add_sensor", "attachment_id", 
            attachment_id)
        attach=_utils.Handle_point(attachment_id)
        Ultrasonic._sensors[sensor] = (runtime_id, attach)

    @_utils.check_type
    @staticmethod
    def get_sensor(sensor: str) -> list:
        if sensor in Ultrasonic._sensors:
            return Ultrasonic._sensors[sensor]
        else:
            raise KeyError(f'Sensor "{sensor}" not found')

    @_utils.check_type
    @staticmethod
    def onSensorUltrasonicEvent(
        sensor: str, 
        compare: str, 
        distance: _Union[int, float], 
        cb: _Callable
    ):
        def cb_wrapper(err, data):
            if err is None:
                cb()
            else:
                _utils.raise_error("onSensorUltrasonicEvent", err, data)

        _utils.check_compare_operator("onSensorUltrasonicEvent", 
            "compare", compare)
        sensor_info = Ultrasonic.get_sensor(sensor)
        _subscribe_event.onSensorUltrasonicEvent(sensor_info[0], 
            sensor_info[1], compare, distance, cb_wrapper)

    @_utils.check_type
    @staticmethod
    def get_obstacle_distance(sensor: str) -> _Union[int, float]:
        _time.sleep(_constants.SLEEP_TIME)
        sensor_info = Ultrasonic.get_sensor(sensor) 
        return _interface.call_api(
            "unity", 
            "unity.sensor.ultrasonicRanging", 
            [sensor_info[0], sensor_info[1]]
        )
    

class Auditory:
    _decibel_val=0

    # 获取声音强度
    @staticmethod
    def get_decibel_value():
        return Auditory._decibel_val

    @_utils.check_type
    @staticmethod
    def onSensorSoundEvent(
        compare: str, 
        decibel_value: _Union[int, float], 
        cb: _Callable
    ):
        def cb_wrapper(err, data):
            if err is None:
                if data and data.get("data") and data["data"] != "":
                    Auditory._decibel_val = data["data"]
                    cb()
            else:
                _utils.raise_error("onSensorSoundEvent", err, data)

        _utils.check_compare_operator("onSensorSoundEvent", 
            "compare", compare)
        decibel_value = max(0, min(decibel_value, 150))
        _subscribe_event.onSensorSoundEvent(compare, decibel_value, cb_wrapper)

    # 开始分贝识别
    @staticmethod
    def start_decibel_recognition():
        def cb_wrapper(err, data):
            if err is None:
                if data and data.get("data") and data["data"] != "":
                    Auditory._decibel_val = data["data"]
            else:
                _utils.raise_error("start_decibel_recognition", err, data)

        _subscribe_event.onSensorSoundEvent("==", "", cb_wrapper)

    # 结束分贝识别
    @staticmethod
    def stop_decibel_recognition():
        _interface.call_api_async(
            "web-ide", 
            "api.openDecibelDetectionPage", 
            [{"type":"end"}]
        )


class Visual:
    _sensors = {}

    # 将传感器绑定到对象的挂点
    @_utils.check_type
    @staticmethod
    def add_sensor(sensor: str, runtime_id: int, attachment_id: tuple):
        runtime_id = max(0, runtime_id)
        _utils.check_attachment_id("add_sensor", "attachment_id", 
            attachment_id)
        attach=_utils.Handle_point(attachment_id)
        Visual._sensors[sensor] = (runtime_id, attach)

    # 获取传感器信息
    @_utils.check_type
    @staticmethod
    def get_sensor(sensor: str):
        if sensor in Visual._sensors:
            return Visual._sensors[sensor]
        else:
            raise KeyError(f'Sensor "{sensor}" not found')

    # 打开或关闭传感器画面
    @_utils.check_type
    @staticmethod
    def open_visual_sensor(sensor: str, action_type: bool=True):
        sensor_info=Visual.get_sensor(sensor)
        if action_type:
            _interface.call_api(
                "unity",
                "unity.sensor.openVision",
                [sensor_info[0], sensor_info[1], sensor]
            )
        else:
            _interface.call_api(
                "unity", 
                "unity.sensor.closeVision", 
                [sensor]
            )

class Temperature:
    _sensors = {}

    @_utils.check_type
    @staticmethod
    def add_sensor(sensor: str, runtime_id: int) -> None:
        runtime_id = max(0, runtime_id)
        Temperature._sensors[sensor] = runtime_id
        _interface.call_api(
            "unity",
            "unity.sensor.attachTemperature",
            [runtime_id]
        )

    @_utils.check_type
    @staticmethod
    def get_sensor(sensor: str) -> int:
        if sensor in Temperature._sensors:
            return Temperature._sensors[sensor]
        else:
            raise KeyError(f'Sensor "{sensor}" not found')

    @_utils.check_type
    @staticmethod
    def onSensorTemperatureEvent(
        sensor: str, 
        compare: str, 
        temperature: _Union[int, float],
        cb: _Callable
    ):
        def cb_wrapper(err, data):
            if err is None:
                cb()
            else:
                _utils.raise_error("onSensorTemperatureEvent", err, data)

        _utils.check_compare_operator("onSensorTemperatureEvent", 
            "compare", compare)
        temperature = max(-40, min(temperature, 120))
        runtime_id = Temperature.get_sensor(sensor)
        _subscribe_event.onSensorTemperatureEvent(runtime_id,
            compare, temperature, cb_wrapper)
    
    # 设置判定区域温度
    @_utils.check_type
    @staticmethod
    def set_temperature(area_id: int, temp_val: _Union[int, float]=0):
        area_id = max(0, area_id)
        temp_val = max(-40, min(temp_val, 120))
        _interface.call_api(
            "unity", 
            "unity.sensor.setTemperature", 
            [area_id, temp_val]
        )

    # 持续检测判定区域温度
    @_utils.check_type
    @staticmethod
    def startTemperatureDetection(area_id: int):
        area_id = max(0, area_id)
        _subscribe_event.startTemperatureDetection(area_id)

    # 获取温度值
    @_utils.check_type
    @staticmethod
    def get_temperature_value(sensor: str):
        _time.sleep(_constants.SLEEP_TIME)
        runtime_id=Temperature.get_sensor(sensor)
        return _interface.call_api(
            "unity",
            "unity.sensor.getTemperature",
            [runtime_id]
        )


class Humidity:
    _sensors = {}

    @_utils.check_type
    @staticmethod
    def add_sensor(sensor: str, runtime_id: int) ->None:
        runtime_id = max(0, runtime_id)
        Humidity._sensors[sensor] = runtime_id
        _interface.call_api(
            "unity",
            "unity.sensor.attachHumidity",
            [runtime_id]
        )

    @_utils.check_type
    @staticmethod
    def get_sensor(sensor: str) -> int:
        if sensor in Humidity._sensors:
            return Humidity._sensors[sensor]
        else:
            raise KeyError(f'Sensor "{sensor}" not found')

    @_utils.check_type
    @staticmethod
    def onSensorHumidityEvent(
        sensor: str, 
        compare: str, 
        humidity_value: _Union[int, float],
        cb: _Callable
    ):
        def cb_wrapper(err, data):
            if err is None:
                cb()
            else:
                _utils.raise_error("onSensorHumidityEvent", err, data)

        _utils.check_compare_operator("onSensorHumidityEvent", 
            "compare", compare)
        humidity_value= max(0, min(humidity_value, 100))
        runtime_id = Humidity.get_sensor(sensor) 
        _subscribe_event.onSensorHumidityEvent(runtime_id, compare,
            humidity_value, cb_wrapper)

    # 设置判定区域湿度
    @_utils.check_type
    @staticmethod
    def set_humidity(area_id: int, humidity_value: _Union[int, float]=0):
        area_id = max(0, area_id)
        humidity_value= max(0, min(humidity_value, 100))
        _interface.call_api(
            "unity",
            "unity.sensor.setHumidity",
            [area_id, humidity_value]
        )

    # 持续检测判定区域湿度
    @_utils.check_type
    @staticmethod
    def startHumidityDetection(area_id: int):
        area_id = max(0, area_id)
        _subscribe_event.startHumidityDetection(area_id)

    @_utils.check_type
    @staticmethod
    def get_humidity_value(sensor: str):
        _time.sleep(_constants.SLEEP_TIME)
        runtime_id=Humidity.get_sensor(sensor)
        return _interface.call_api(
            "unity",
            "unity.sensor.getHumidity",
            [runtime_id]
        )


class Gravity:
    _sensors = {}

    @_utils.check_type
    @staticmethod
    def add_sensor(sensor: str, area_id: int) -> None:
        area_id = max(0, area_id)
        Gravity._sensors[sensor] = area_id
        _interface.call_api(
            "unity",
            "unity.sensor.attachGravity",
            [area_id]
        )

    @_utils.check_type
    @staticmethod
    def get_sensor(sensor: str) -> int:
        if sensor in Gravity._sensors:
            return Gravity._sensors[sensor]
        else:
            raise KeyError(f'Sensor "{sensor}" not found')

    @_utils.check_type
    @staticmethod
    def onSensorGravityEvent(
        sensor: str, 
        compare: str, 
        gravity_value: _Union[int, float],
        cb: _Callable
    ):
        def cb_wrapper(err, data):
            if err is None:
                cb()
            else:
                _utils.raise_error("onSensorGravityEvent", err, data)

        _utils.check_compare_operator("onSensorGravityEvent", 
            "compare", compare)
        gravity_value= max(0, min(gravity_value, 10000))
        runtime_id = Gravity.get_sensor(sensor)
        _subscribe_event.onSensorGravityEvent(runtime_id, compare,
            gravity_value, cb_wrapper)

    # 设置对象重力
    @_utils.check_type
    @staticmethod
    def set_gravity(runtime_id: int, gravity_value: _Union[int, float]=0):
        runtime_id = max(0, runtime_id)
        gravity_value= max(0, min(gravity_value, 10000))
        _interface.call_api(
            "unity",
            "unity.sensor.setGravity",
            [runtime_id, gravity_value]
        )

    # 获取重力值
    @_utils.check_type
    @staticmethod
    def get_gravity_value(sensor: str):
        _time.sleep(_constants.SLEEP_TIME)
        runtime_id = Gravity.get_sensor(sensor) 
        return _interface.call_api(
            "unity",
            "unity.sensor.getGravity",
            [runtime_id]
        )




import uuid
import threading
from crealand._core.bridge._interface import call_api_async
from crealand._core.websocket.websocket_client import get_ws_client, get_callback
from crealand._utils._logger_setup import setup_logger

logger = setup_logger()

def _subscribeEvent(dest, func_name, func_args, subscribe_id, callback=None):
    def eventCallback(err, data):
        try:
            logger.info(f'Trigger the event callback. err: {err}, data: {data}')
            if 'type' in data and 'result' in data:
                logger.info('This is an event trigger.')
                if data['type'] == 'trigger' and data['result']:
                    logger.info('The type is trigger.')
                    callback(err, data)
                elif data['type'] == 'callback' and data['result']:
                    logger.info('The type is callback.')
                    ws_client = get_ws_client()
                    call_api_async(
                        dest='web-ide', 
                        func_name='event.addTaskSubscribe', 
                        func_args=[
                            dest, 
                            ws_client.session_id, 
                            ws_client.task_id, 
                            subscribe_id
                        ], 
                        callback=None
                    )
            else:
                logger.info('This is an api trigger.')
                callback(err, data)
            logger.info('Finished!')
        except Exception as e:
            logger.error(f'An error occurred: {e}')

    return call_api_async(dest, func_name, func_args, eventCallback if callback else None)

def onBroadcastEvent(info, callback):
    event_callback = get_callback()
    event_callback.registerBroadcast(info, callback)

def sendBroadcast(info):
    event_callback = get_callback()
    threading.Thread(
        target=event_callback.broadcast, 
        args=(info, )
    ).start()

# Crealand IDE-WEB
def onAIFigureEvent(number, callback):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        'AIFigureEvent', {
            'subscribeId': subscribe_id, 
            'targets': [{
                'name': 'number', 
                'type': 'and', 
                'conditions': {
                    '==': number
                }}
            ]
        }
    ]
    return _subscribeEvent(
        dest='web-ide', 
        func_name='event.subscribeRemoteEvent', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )

def onAIGestureEvent(direction, callback):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        'AIGestureEvent', {
            'subscribeId': subscribe_id, 
            'targets': [{
                'name': 'direction', 
                'type': 'and', 
                'conditions': {
                    '==': direction
                }}
            ]
        }
    ]
    return _subscribeEvent(
        dest='web-ide', 
        func_name='event.subscribeRemoteEvent', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )

def onAIAsrEvent(text, callback):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        'AIAsrEvent', {
            'subscribeId': subscribe_id, 
            'targets': [{
                'name': 'text', 
                'type': 'and', 
                'conditions': {
                    '==': text
                }}
            ]
        }
    ]
    return _subscribeEvent(
        dest='web-ide', 
        func_name='event.subscribeRemoteEvent', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )

def onSensorSoundEvent(compare, decibel_value, callback):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        'SensorSoundEvent', {
            'subscribeId': subscribe_id, 
            'targets': [{
                'name': 'decibel_value', 
                'type': 'and', 
                'conditions': {
                    compare: decibel_value
                }}
            ]
        }
    ]
    return _subscribeEvent(
        dest='web-ide', 
        func_name='event.subscribeRemoteEvent', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )

# Crealand IDE-3D
def onKeyEvent(action, button, callback):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        button, 
        action, {
            'subscribeId': subscribe_id, 
            'targets': []
        }
    ]
    return _subscribeEvent(
        dest='unity', 
        func_name='unity.input.onEventKeyCode', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )

def onAreaObjectEvent(runtime_id, action, area_id, callback):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        runtime_id, 
        action, 
        area_id, {
            'subscribeId': subscribe_id, 
            'targets': []
        }
    ]
    return _subscribeEvent(
        dest='unity', 
        func_name='unity.editableTrigger.onEventRuntimeIdTrigger', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )

def onAreaClassEvent(config_id, action, area_id, callback):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        config_id, 
        action, 
        area_id, {
            'subscribeId': subscribe_id, 
            'targets': []
        }
    ]
    return _subscribeEvent(
        dest='unity', 
        func_name='unity.editableTrigger.onEventConfigIdTrigger', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )

def onSensorUltrasonicEvent(runtime_id, attachment_id, compare, distance, callback):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        runtime_id, 
        attachment_id, {
            'subscribeId': subscribe_id, 
            'targets': [{
                'name': 'distance', 
                'type': 'and', 
                'conditions': {
                    compare: distance
                }}
            ]
        }
    ]
    return _subscribeEvent(
        dest='unity', 
        func_name='unity.sensor.onEventUltrasonicRanging', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )

def onSensorTemperatureEvent(temperature_sensor, compare, temperature, callback):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        temperature_sensor, {
            'subscribeId': subscribe_id, 
            'targets': [{
                'name': 'temperature', 
                'type': 'and', 
                'conditions': {
                    compare: temperature
                }}
            ]
        }
    ]
    return _subscribeEvent(
        dest='unity', 
        func_name='unity.sensor.onEventTemperature', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )

def onSensorHumidityEvent(humidity_sensor, compare, humidity_value, callback):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        humidity_sensor, {
            'subscribeId': subscribe_id, 
            'targets': [{
                'name': 'humidity_value', 
                'type': 'and', 
                'conditions': {
                    compare: humidity_value
                }}
            ]
        }
    ]
    return _subscribeEvent( 
        dest='unity', 
        func_name='unity.sensor.onEventHumidity', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )

def onSensorGravityEvent(gravity_sensor, compare, gravity_value, callback):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        gravity_sensor, {
            'subscribeId': subscribe_id, 
            'targets': [{
                'name': 'gravity_value', 
                'type': 'and', 
                'conditions': {
                    compare: gravity_value
                }}
            ]
        }
    ]
    return _subscribeEvent(
        dest='unity', 
        func_name='unity.sensor.onEventGravity', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )

def startTemperatureDetection(judge_area_id, callback=None):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        judge_area_id, {
            'subscribeId': subscribe_id, 
            'targets': []
        }
    ]
    return _subscribeEvent(
        dest='unity', 
        func_name='unity.sensor.startTemperatureDetection', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )

def startHumidityDetection(judge_area_id, callback=None):
    subscribe_id = str(uuid.uuid4())
    func_args = [
        judge_area_id, {
            'subscribeId': subscribe_id, 
            'targets': []
        }
    ]
    return _subscribeEvent(
        dest='unity', 
        func_name='unity.sensor.startHumidityDetection', 
        func_args=func_args, 
        subscribe_id=subscribe_id, 
        callback=callback
    )




from setuptools import setup, find_packages

name = 'crealand'
# 正式版本号
version = '1.0.46'
# 测试版本号
# version = '1.0.9'
description = 'Python SDK for Crealand'
author = 'UBTECH Robotics'
author_email = 'swenggroup@ubtrobot.com'

package_dir = {
    'crealand': 'crealand'
}

package_data = {
}

install_requires = [
    'websockets'
]

# require_data_files = ['requirements.txt']
require_data_files = []
example_data_files = [
]
doc_data_files = [
]

data_files = []
try:
    data_files.extend(require_data_files)
    data_files.extend(example_data_files)
    data_files.extend(doc_data_files)
except:
    pass

setup(
    name=name,
    version=version,
    description=description,
    author=author,
    author_email=author_email,
    packages=find_packages(exclude=["tests*"]),
    package_dir=package_dir,
    package_data=package_data,
    data_files=data_files,
    install_requires=install_requires
)



from crealand._core.bridge import _interface
from crealand._utils import _utils

class Sound:
    # 播放声音
    @_utils.check_type
    @staticmethod
    def play(runtime_id: int, res_id: str,is_loop: bool=False):
        runtime_id = max(0, runtime_id)
        _interface.call_api_async(
            "unity",
            "unity.sound.playSound",
            [runtime_id, is_loop, res_id],
        )

    # 声音
    @staticmethod
    @_utils.check_type
    def adjust_volume(runtime_id: int, volume: int = 50):
        runtime_id = max(0, runtime_id)
        volume = max(0, min(volume, 100))
        _interface.call_api_async(
            "unity",
            "unity.sound.adjustVolume",
            [runtime_id, volume],
        )

    # 停止播放
    @_utils.check_type
    @staticmethod
    def stop(runtime_id: int):
        runtime_id = max(0, runtime_id)
        _interface.call_api_async(
            "unity", 
            "unity.sound.stopSound", 
            [runtime_id]
        )

    # 设置背景音效
    @_utils.check_type
    @staticmethod
    def play_bgm(res_id: str):
        _interface.call_api_async(
            "unity", 
            "unity.sound.playBgSound", 
            [res_id]
        )

    # 背景音效音量
    @_utils.check_type
    @staticmethod
    def adjust_bgm_volume(volume: int = 50):
        volume = max(0, min(volume, 100))
        _interface.call_api_async(
            "unity",
            "unity.sound.adjustBgVolume",
            [volume],
        )

    # 停止背景音效
    @staticmethod
    def stop_bgm():
        _interface.call_api_async(
            "unity",
            "unity.sound.stopBgSound",
            [],
        )


class Video:
    # 播放视频
    @_utils.check_type
    @staticmethod
    def play(res_id: str):
        _interface.call_api(
            "web-ide",
            "api.playVideo",
            [{"pythonOssId":res_id}],
        )


class Image:
    # 播放图片
    @_utils.check_type
    @staticmethod
    def show(res_id: str):
        _interface.call_api(
            "web-ide",
            "api.showImage",
            [{"pythonOssId":res_id}],
        )
